import BaseController, { ROLE_ANON } from 'common/base.controller'
import { ZError } from 'common/ZError'
import { role, router } from 'decorators/router'
import logger from 'logger/logger'
import { AuthRecord, PlatEnum } from 'modules/AuthRecord'
import { DiscordSvr, getAvableAccessToken, userGuildMember } from 'services/discord.svr'
import { hmacsha256 } from 'utils/security.util'

const checkSign = (params: { address?: string; sign?: string }) => {
  let { address, sign } = params
  address = address.toLowerCase()
  if (!address || !sign) {
    throw new ZError(10, 'invalid params')
  }
  let signCheck = hmacsha256(`address=${address}`, process.env.HASH_SALT)
  if (sign !== signCheck) {
    throw new ZError(11, 'invalid sign')
  }
}
class MainController extends BaseController {
  /**
   * Refresh token
   */
  @role(ROLE_ANON)
  @router('get /activity/202306/:address')
  async getToken(req, res) {
    let { address, sign } = req.params
    if (!address || !sign) {
      throw new ZError(10, 'invalid params')
    }
    let signCheck = hmacsha256(`address=${address}`, process.env.HASH_SALT)
    if (sign !== signCheck) {
      throw new ZError(11, 'invalid sign')
    }
    address = address.toLowerCase()
    let records = await AuthRecord.find({ address })
    let result: any = {
      discord: {},
      twitter: {},
    }
    let discordRecord
    for (const record of records) {
      switch (record.platform) {
        case 4:
          result.twitter = {
            id: record.openId,
            username: record.username,
            avatar: record.avatar,
          }
          break
        case 7:
          result.discord = {
            id: record.openId,
            username: record.username,
            discriminator: record.discriminator,
            verified: record.condition,
          }
          discordRecord = record
          if (!record.condition) {
            await record.updateOne({ $inc: { version: 1, condition: 1 } })
          }

          break
      }
    }
    // if (result.discord.id && !result.discord.verified) {
    //   setImmediate(async () => {
    //     try {
    //       let verified = await new DiscordSvr().checkUserRole(result.discord.id)
    //       if (verified) {
    //         logger.info('check role with verified: ' + verified)
    //         await AuthRecord.updateOne({ openId: result.discord.id, platform: 7, address }, { $set: { condition: 1 } })
    //       }
    //     } catch (err) {
    //       logger.info('check role with err: ' + err.message || err)
    //     }
    //   })
    // }
    return result
  }

  @role(ROLE_ANON)
  @router('get /activity/202306/check/:id')
  async checkRole(req, res) {
    let { id } = req.params
    let verified = await new DiscordSvr().checkUserRole(id)
    return { verified }
  }

  @role(ROLE_ANON)
  @router('get /activity/twitter/:address')
  async checkTwitterFollow(req) {
    let { address } = req.params
    checkSign(req.params)
    address = address.toLowerCase()
    let record = await AuthRecord.findOne({ address, platform: PlatEnum.TWITTER })
    let result: any = {}
    if (!!record) {
      result.username = record.username
      result.userid = record.openId
      result.avatar = record.avatar
    }
    return result
  }

  @role(ROLE_ANON)
  @router('get /activity/discord/:address')
  async checkDiscord(req) {
    let { address } = req.params
    checkSign(req.params)
    address = address.toLowerCase()
    let record = await AuthRecord.findOne({ address, platform: PlatEnum.DISCORD })
    let result: any = {}
    if (!!record) {
      result.verified = record.condition
      result.username = record.username
      result.userid = record.openId
      result.discriminator = record.discriminator
    }
    return result
  }

  /**
   * 检查是否加入某guid
   */
  @role(ROLE_ANON)
  @router('post /activity/discord/guild')
  async checkDiscordJoin(req) {
    let { gid, address } = req.params
    checkSign(req.params)
    address = address.toLowerCase()
    let record = await AuthRecord.findOne({ address, platform: PlatEnum.DISCORD })
    if (!record) {
      throw new ZError(12, 'not found discord record')
    }
    let accessToken = await getAvableAccessToken(record)
    // TODO:: check local cache
    let data = await userGuildMember(accessToken, gid)
    if (!data || data.code) {
      throw new ZError(13, 'not found guild')
    }
    return { result: true }
  }
  /**
   * 检查是否有某个role
   */
  @role(ROLE_ANON)
  @router('post /activity/discord/role')
  async checkDiscordRole(req) {
    let { rid, gid, address } = req.params
    checkSign(req.params)
    address = address.toLowerCase()
    let record = await AuthRecord.findOne({ address, platform: PlatEnum.DISCORD })
    if (!record) {
      throw new ZError(20, 'need connect discord first')
    }
    let accessToken = await getAvableAccessToken(record)
    console.log('discord access token:', accessToken)
    // TODO:: check local cache
    let data = await userGuildMember(accessToken, gid)
    console.log(data?.roles)
    if (!data || data.code) {
      throw new ZError(21, 'have not join guild')
    }
    let roleSet = new Set(data.roles)
    return { result: roleSet.has(rid) }
  }
}
